/***
 *
 * Examples given in [LeJeRi2012].
 * 
 * Bibliography
 * 
 * [LeRi2012] "Explicit Galois Obstruction and Descent for Hyperelliptic
 * Curves with Tamely Cyclic Reduced Automorphism Group", Preprint, 2012
 *
 *********************************************************************/

//To run  :  get the G3Twists package (version 1.1) from
// http://perso.univ-rennes1.fr/reynald.lercier/file/g3twists_v1.1.tar.gz
//Install this in the appropriate directory, so that the following runs:

AttachSpec("g3twists_v1-1/package/spec");
load "ADI.m";
load "Descent.m";

// (1) Genus 5, non-descending curve in form (2):

//Creating split algebra:
d1 := 2;
d2 := 3;

F := RationalsAsNumberField();
R<t> := PolynomialRing(F);
K1<r1> := NumberField(t^2 - d1);
R<t> := PolynomialRing(K1);
L<r> := NumberField(t^2 - (r1 + 2));
L := AbsoluteField(L);
r := L.1;

//Coefficients such the curve is isomorphic to its conjugate:
a := 7 + r1;
b := 3 - 2*r1;
c := 1 + r1;
d := 12*r1;
e := -d2*(1 - r1);
f := -d2^2*(3 + 2*r1);
g := -d2^3*(7 - r1);

//Creation of the curve:
S<x,z> := PolynomialRing(L,2);
f := a*x^12 + b*x^10*z^2 + c*x^8*z^4 + d*x^6*z^6 + e*x^4*z^8 + f*x^2*z^10 + g*z^12;
//Messing it up further:
T := Matrix(L,2,2,[(L.1 + 1), (2*L.1 + 3), 1, 5]);
h := hom<S -> S | [T[1,1]*x + T[1,2]*z, T[2,1]*x + T[2,2]*z]>;
f := (L.1 + 2)*h(f);

//Manual input of automorphism:
n0 := 2;
T := T^(-1);
alpha := T*Matrix(L,2,2,[-1,0,0,1])*T^(-1);
fL := StandardForm(f,alpha);
C := CaseDistinction(fL,n0);
as := FundamentalCoefficients(fL,C);

//Calculation of the normalized invariants:
I,WI,J,WJ,K,k := GenericNormalizedADI(as,C);
J;
WJ;
//This indeed lives over a quadratic field.
I;
WI;
//We are in the rationals, as expected.

//Partial descent:
f1,as1 := GenericPartialDescent(as,C,J,WJ,K);
f1;
as1;

//So far so good. Let us try to descend further. In this case there exists a descent if and only if there exists a hyperelliptic descent.
g := FunctionFromPartialDescent(as1,C,K,k);
r := AutomorphismFromPartialDescent(as1,C,K,k);
test,f0 := HyperellipticDescentFromDivAndAut(as1,C,g,r,K,k);
test;



//(2) Genus 3, non-descending curve in form (2):

//Constructing a non-splitting algebra:
d1 := 2;
d2 := 3;

F := RationalsAsNumberField();
R<t> := PolynomialRing(F);
K1<r1> := NumberField(t^2 - d1);
R<t> := PolynomialRing(K1);
L<r> := NumberField(t^2 - (r1 + 2));
L := AbsoluteField(L);
r := L.1;

//Coefficients such the curve is isomorphic to its conjugate:
a := 7 + r1;
b := 3 - 2*r1;
c := 12*r1;
d := -d2*(3 + 2*r1);
e := -d2^2*(7 - r1);

//Creation of the curve:
S<x,z> := PolynomialRing(L,2);
PL<X> := PolynomialRing(L);
Dhom := hom<S -> PL | [X,1]>;
f := a*x^8 + b*x^6*z^2 + c*x^4*z^4 + d*x^2*z^6 + e*z^8;
WPSNf := ShiodaInvariants(Dhom(f) : normalize := true);
//Messing it up further:
h := hom<S -> S | [(L.1 + 1)*x + (2*L.1 + 3)*z, x + 5*z]>;
f := (L.1 + 2)*h(f);

//Detecting automorphisms by Shioda invariants and transformation to normal form.
test,fK,alpha,n0 := HasCyclicAutomorphismGroup(f);
fL := StandardForm(fK,alpha);
C := CaseDistinction(fL,n0);
as := FundamentalCoefficients(fL,C);

//Calculation of the normalized invariants:
I,WI,J,WJ,K,k := GenericNormalizedADI(as,C);
J;
WJ;
//This indeed lives over a quadratic field.
I;
WI;
//We are in the rationals, as expected.

//Partial descent:
f1,as1 := GenericPartialDescent(as,C,J,WJ,K);
f1;
as1;

//Intermediate test:
WPSNf1 := ShiodaInvariants(Dhom(f1) : normalize := true);
WPSNf;
WPSNf1;

//So far so good. Let us try to descend further. In this case there exists a descent if and only if there exists a hyperelliptic descent. Which is not the case:
g := FunctionFromPartialDescent(as1,C,K,k);
r := AutomorphismFromPartialDescent(as1,C,K,k);
test,f0 := HyperellipticDescentFromDivAndAut(as1,C,g,r,K,k);
test;

//Automated version:
Descent(f);


//(3) Genus 3, descending curve in form (2):

//Constructing a splitting algebra:
d1 := 3;
d2 := 13;

F := RationalsAsNumberField();
R<t> := PolynomialRing(F);
K1<r1> := NumberField(t^2 - d1);
R<t> := PolynomialRing(K1);
L<r> := NumberField(t^2 - (r1 + 2));
L := AbsoluteField(L);
r := L.1;

//Coefficients such the curve is isomorphic to its conjugate:
a := 7 + r1;
b := 3 - 2*r1;
c := 12*r1;
d := -d2*(3 + 2*r1);
e := -d2^2*(7 - r1);

//Creation of the curve:
S<x,z> := PolynomialRing(L,2);
PL<X> := PolynomialRing(L);
Dhom := hom<S -> PL | [X,1]>;
f := a*x^8 + b*x^6*z^2 + c*x^4*z^4 + d*x^2*z^6 + e*z^8;
WPSNf := ShiodaInvariants(Dhom(f) : normalize := true);

//Messing it up further:
h := hom<S -> S | [(L.1 + 1)*x + (2*L.1 + 3)*z, x + 5*z]>;
f := (L.1 + 2)*h(f);

//Detecting automorphisms by Shioda invariants and transformation to normal form.
test,fK,alpha,n0 := HasCyclicAutomorphismGroup(f);
fL := StandardForm(fK,alpha);
C := CaseDistinction(fL,n0);
as := FundamentalCoefficients(fL,C);

//Calculation of the normalized invariants:
I,WI,J,WJ,K,k := GenericNormalizedADI(as,C);
J;
WJ;
//This indeed lives over a quadratic field.
I;
WI;
//We are in the rationals, as expected.

//Partial descent:
f1,as1 := GenericPartialDescent(as,C,J,WJ,K);
f1;
as1;

//Intermediate test:
WPSNf1 := ShiodaInvariants(Dhom(f1) : normalize := true);
WPSNf;
WPSNf1;

//So far so good. Let us try to descend further. In this case there exists a descent if and only if there exists a hyperelliptic descent.

g := FunctionFromPartialDescent(as1,C,K,k);
r := AutomorphismFromPartialDescent(as1,C,K,k);

//Manually:
S<x,z> := Parent(g);
R<t> := PolynomialRing(Rationals());
w0 := Roots(t^2 - 3,K)[2][1];
w := 1/w0;
d := k ! (w^2);
CNorm := Conic([1,-d,-r]);
test,P := HasRationalPoint(CNorm);
w0;
lambda := (P[1]/P[3]) + (P[2]/P[3])*w;
lambda;

g0 := HyperellipticDescendedDivisorFromDivAndAut(as1,C,g,r,K,k,w,P);
g;
g0;

//Pulling pack the divisor:
p,q := NTo1CoverFromDivAndAut(as1,C,g,r,K,k,w,P);
h := hom<S -> S | [p,q]>;
f0 := h(g0);

S0<x,z> := PolynomialRing(k,2);
f0 := (S0 ! f0);
f0 := f0 / Coefficients(f0)[1];
f0;

//We get a descent! Check of the invariants:
WPSNf0 := ShiodaInvariants(Dhom(f0) : normalize := true);
WPSNf;
WPSNf0;

//Quicker:
test,f0 := HyperellipticDescentFromDivAndAut(as1,C,g,r,K,k);

//Automated version:
Descent(f);

//(4) Genus 5, non-hyperelliptic descent for a curve in form (2):

//Constructing a non-splitting algebra:
d1 := 2;
d2 := 3;

F := RationalsAsNumberField();
R<t> := PolynomialRing(F);
K1<r1> := NumberField(t^2 - d1);
R<t> := PolynomialRing(K1);
L<r> := NumberField(t^2 - (r1 + 2));
L := AbsoluteField(L);
r := L.1;

//Coefficients such the curve is isomorphic to its conjugate:
a := 7 + r1;
b := 3 - 2*r1;
c := 12*r1;
d := -d2*(3 + 2*r1);
e := -d2^2*(7 - r1);

//Creation of the curve:
S<x,z> := PolynomialRing(L,2);
f := a*x^12 + b*x^9*z^3 + c*x^6*z^6 + d*x^3*z^9 + e*z^12;
//Messing it up further:
T := Matrix(L,2,2,[(L.1 + 1), (2*L.1 + 3), 1, 5]);
h := hom<S -> S | [T[1,1]*x + T[1,2]*z, T[2,1]*x + T[2,2]*z]>;
f := (L.1 + 2)*h(f);

//Manual input of automorphism:
n0 := 3;
T := T^(-1);
//Cheating!
alpha := T*Matrix(L,2,2,[-1,0,0,1])*T^(-1);
fL := StandardForm(f,alpha);
C := CaseDistinction(fL,n0);
as := FundamentalCoefficients(fL,C);

//Calculation of the normalized invariants:
I,WI,J,WJ,K,k := GenericNormalizedADI(as,C);
J;
WJ;
//This indeed lives over a quadratic field.
I;
WI;
//We are in the rationals, as expected.

//Partial descent:
f1,as1 := GenericPartialDescent(as,C,J,WJ,K);
f1;
as1;

//No hyperelliptic descent:
g := FunctionFromPartialDescent(as1,C,K,k);
r := AutomorphismFromPartialDescent(as1,C,K,k);

test,f0 := HyperellipticDescentFromDivAndAut(as1,C,g,r,K,k);
test;

//But a non-hyperelliptic descent does exist:
//Manually:
S<x,z> := Parent(g);
R<t> := PolynomialRing(Rationals());
w := Roots(t^2 - 2,K)[1][1];

Eq0,D0 := NonHyperellipticDescendedDivisorFromDivAndAut(as1,C,g,r,K,k,w);
Eq0,D0;

//Quicker:
Eq0,D0 := NonHyperellipticDescentFromDivAndAut(as1,C,g,r,K,k);
Eq0;
D0;



//(5) Genus 5, descending curve in form (3):

F := RationalsAsNumberField();
R<t> := PolynomialRing(F);
L<r> := NumberField(t^2 - 5);
S<x,z> := PolynomialRing(L,2);
f0 := z*(x^9 + 2*x^6*z^3 + 3*x^3*z^6 + 4*z^9);
T := Matrix(L,2,2,[(r + 1), (r - 2), r, -1]);
h := hom<S -> S | [T[1,1]*x + T[1,2]*z, T[2,1]*x + T[2,2]*z]>;
f := (r + 1)*h(f0);
f;

//Manual input of automorphism:
n0 := 3;
T := T^(-1);
alpha := T*Matrix(L,2,2,[-1,0,0,1])*T^(-1);
fL := StandardForm(f,alpha);
fL := NormalizeCaseI1(fL);

C := CaseDistinction(fL,n0);
as := FundamentalCoefficients(fL,C);

//Calculation of the normalized invariants:
I,WI,J,WJ,K,k := GenericNormalizedADI(as,C);
J;
WJ;
//This indeed lives over a quadratic field.
I;
WI;
//We are in the rationals, as expected.

//Partial descent:
f1,as1 := GenericPartialDescent(as,C,J,WJ,K);
f1;
as1;

//Done:
I;
I1 := GenericNormalizedADI(as1,C);
I1;


//(6) Genus 3, descending curve in form (4):

F := RationalsAsNumberField();
R<t> := PolynomialRing(F);
L<r> := NumberField(t^2 - 5);
S<x,z> := PolynomialRing(L,2);
PL<X> := PolynomialRing(L);
Dhom := hom<S -> PL | [X,1]>;
f0 := x*z*(x^6 + 3*x^4*z^2 + 2*x^2*z^4 + 4*z^6);
h := hom<S -> S | [(r + 1)*x + (r - 2)*z, r*x - z]>;
f := (r + 1)*h(f0);
f;

//Invariants:
SIf,WPSNf := ShiodaInvariants(Dhom(f) : normalize := true);

//Detecting automorphisms by Shioda invariants and transformation to normal form.
test,fK,alpha,n0 := HasCyclicAutomorphismGroup(f);
fL := StandardForm(fK,alpha);
C := CaseDistinction(fL,n0);
as := FundamentalCoefficients(fL,C);

//Calculation of the normalized invariants:
I,WI,J,WJ,K,k := GenericNormalizedADI(as,C);
J;
WJ;
//This indeed lives over a quadratic field.
I;
WI;
//We are in the rationals, as expected.

//Partial descent:
f1,as1 := GenericPartialDescent(as,C,J,WJ,K);
f1;
as1;

//Done:
WPSNf1:= ShiodaInvariants(Dhom(f1) : normalize := true);
WPSNf;
WPSNf1;

//Automated version:
Descent(f);

quit;
